@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2020. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -321,12 +321,16 @@ handle_v3_message(Mgr, UdpId, Ip, UdpPort, AgentIp,
 
         catch
             throw:{error, Reason, B}:_ ->
-                udp_send(UdpId, AgentIp, UdpPort, B),
+                {_, Pid} = Mgr,
                 ?EPRINT("Decoding (v3) error - Auto-sending Report:"
-                        "~n   Reason: ~p"
-                        "~n   Port:   ~p"
-                        "~n   Ip:     ~p",
-                        [Reason, UdpPort, Ip]),
+                        "~n   Reason:    ~p"
+                        "~n   Port:      ~p"
+                        "~n   Ip:        ~p"
+                        "~n   (mgr) Pid: ~p",
+                        [Pid, Reason, UdpPort, Ip]),
+                udp_send(UdpId, AgentIp, UdpPort, B),
+                %% Can we be sure that this error is not expected?
+                Pid ! {error, Reason},
                 [];
 
             throw:{error, Reason}:_ ->
