@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2013-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2013-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -208,16 +208,16 @@ initial_hello({call, From}, {start, Timeout},
                                      session_cache_cb = CacheCb},
             protocol_specific = PS,
             handshake_env = #handshake_env{renegotiation = {Renegotiation, _}},
-            connection_env = CEnv,
+            connection_env = #connection_env{cert_key_pairs = CertKeyPairs} = CEnv,
 	    ssl_options = #{versions := Versions} = SslOpts,
-	    session = #session{own_certificates = OwnCerts} = NewSession,
+	    session = Session0,
 	    connection_states = ConnectionStates0
 	   } = State0) ->
     Packages = maps:get(active_n, PS),
     dtls_socket:setopts(Transport, Socket, [{active,Packages}]),
-    Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, NewSession), 
+    Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, Session0, CertKeyPairs),
     Hello = dtls_handshake:client_hello(Host, Port, ConnectionStates0, SslOpts,
-					Session#session.session_id, Renegotiation, OwnCerts),
+					Session#session.session_id, Renegotiation),
 
     MaxFragEnum = maps:get(max_frag_enum, Hello#client_hello.extensions, undefined),
     ConnectionStates1 = ssl_record:set_max_fragment_length(MaxFragEnum, ConnectionStates0),
@@ -303,13 +303,13 @@ hello(internal, #hello_verify_request{cookie = Cookie},
              connection_env = CEnv,
              ssl_options = #{ocsp_stapling := OcspStaplingOpt,
                              ocsp_nonce := OcspNonceOpt} = SslOpts,
-             session = #session{own_certificates = OwnCerts,
-                                session_id = Id},
-             connection_states = ConnectionStates0
+             session = #session{session_id = Id},
+             connection_states = ConnectionStates0,
+	     protocol_specific = PS
             } = State0) ->
     OcspNonce = tls_handshake:ocsp_nonce(OcspNonceOpt, OcspStaplingOpt),
     Hello = dtls_handshake:client_hello(Host, Port, Cookie, ConnectionStates0,
-					SslOpts, Id, Renegotiation, OwnCerts, OcspNonce),
+					SslOpts, Id, Renegotiation, OcspNonce),
     Version = Hello#client_hello.client_version,
     State1 = prepare_flight(State0#state{handshake_env =  
                                              HsEnv#handshake_env{tls_handshake_history
@@ -319,18 +319,17 @@ hello(internal, #hello_verify_request{cookie = Cookie},
     
     {State2, Actions} = dtls_gen_connection:send_handshake(Hello, State1), 
 
-    State = State2#state{connection_env = CEnv#connection_env{negotiated_version = Version} % RequestedVersion
+    State = State2#state{connection_env = CEnv#connection_env{negotiated_version = Version}, % RequestedVersion
+			 protocol_specific = PS#{current_cookie_secret => Cookie}
                         },
     dtls_gen_connection:next_event(?FUNCTION_NAME, no_record, State, Actions);
 hello(internal, #client_hello{extensions = Extensions} = Hello,
-      #state{ssl_options = #{handshake := hello},
-             handshake_env = HsEnv,
+      #state{handshake_env = #handshake_env{continue_status = pause},
              start_or_recv_from = From} = State0) ->
     try tls_dtls_connection:handle_sni_extension(State0, Hello) of
         #state{} = State ->
-            {next_state, user_hello, State#state{start_or_recv_from = undefined,
-                                                 handshake_env = HsEnv#handshake_env{hello = Hello}},
-             [{reply, From, {ok, Extensions}}]}
+            {next_state, user_hello, State#state{start_or_recv_from = undefined},
+             [{postpone, true}, {reply, From, {ok, Extensions}}]}
     catch throw:#alert{} = Alert ->
             alert_or_reset_connection(Alert, ?FUNCTION_NAME, State0)
     end;
@@ -353,16 +352,11 @@ hello(internal, #client_hello{cookie = Cookie} = Hello, #state{static_env = #sta
                     hello(internal, Hello#client_hello{cookie = <<>>}, State)
             end
     end;
-hello(internal, #server_hello{extensions = Extensions} = Hello, 
-      #state{ssl_options = #{
-                 handshake := hello},
-             handshake_env = HsEnv,
+hello(internal, #server_hello{extensions = Extensions}, 
+      #state{handshake_env = #handshake_env{continue_status = pause},
              start_or_recv_from = From} = State) ->
-    {next_state, user_hello, State#state{start_or_recv_from = undefined,
-                                         handshake_env = HsEnv#handshake_env{
-                                                           hello = Hello}},
-     [{reply, From, {ok, Extensions}}]};       
-
+    {next_state, user_hello, State#state{start_or_recv_from = undefined},
+     [{postpone, true},{reply, From, {ok, Extensions}}]};
 hello(internal, #server_hello{} = Hello,
       #state{
          static_env = #static_env{role = client},
@@ -515,22 +509,22 @@ connection(internal, #hello_request{}, #state{static_env = #static_env{host = Ho
                                                                        session_cache_cb = CacheCb
                                                                       },
                                               handshake_env = #handshake_env{renegotiation = {Renegotiation, _}},
-                                              connection_env = CEnv,
-                                              session = #session{own_certificates = OwnCerts} = Session0,
+                                              connection_env = #connection_env{cert_key_pairs = CertKeyPairs} = CEnv,
+                                              session = Session0,
                                               ssl_options = #{versions := Versions} = SslOpts,
                                               connection_states = ConnectionStates0,
                                               protocol_specific = PS
                                              } = State0) ->
-    
-    Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, Session0),
-    Hello = dtls_handshake:client_hello(Host, Port, ConnectionStates0, SslOpts,
-					Session#session.session_id, Renegotiation, OwnCerts),
+    #{current_cookie_secret := Cookie} = PS,
+    Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, Session0, CertKeyPairs),
+    Hello = dtls_handshake:client_hello(Host, Port, Cookie, ConnectionStates0, SslOpts,
+					Session#session.session_id, Renegotiation, undefined),
     Version = Hello#client_hello.client_version,
     HelloVersion = dtls_record:hello_version(Version, Versions),
     State1 = prepare_flight(State0),
-    {State2, Actions} = 
-        dtls_gen_connection:send_handshake(Hello, 
-                                           State1#state{connection_env = 
+    {State2, Actions} =
+        dtls_gen_connection:send_handshake(Hello,
+                                           State1#state{connection_env =
                                                             CEnv#connection_env{negotiated_version = HelloVersion}}),
     State = State2#state{protocol_specific = PS#{flight_state => dtls_gen_connection:initial_flight_state(DataTag)},
                          session = Session},
@@ -680,14 +674,14 @@ handle_client_hello(#client_hello{client_version = ClientVersion} = Hello, State
                handshake_env = #handshake_env{kex_algorithm = KeyExAlg,
                                               renegotiation = {Renegotiation, _},
                                               negotiated_protocol = CurrentProtocol} = HsEnv,
-               connection_env = CEnv,
-               session = #session{own_certificates = OwnCerts} = Session0,
+               connection_env = #connection_env{cert_key_pairs = CertKeyPairs} = CEnv,
+               session = Session0,
                ssl_options = SslOpts} =
             tls_dtls_connection:handle_sni_extension(State0, Hello),
         SessionTracker = proplists:get_value(session_id_tracker, Trackers),
         {Version, {Type, Session}, ConnectionStates, Protocol0, ServerHelloExt, HashSign} =
             dtls_handshake:hello(Hello, SslOpts, {SessionTracker, Session0,
-                                                  ConnectionStates0, OwnCerts, KeyExAlg}, Renegotiation),
+                                                  ConnectionStates0, CertKeyPairs, KeyExAlg}, Renegotiation),
         Protocol = case Protocol0 of
                        undefined -> CurrentProtocol;
                        _ -> Protocol0
