@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2021. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -111,7 +111,8 @@
 %% Setup
 -export([init/1]).
 
--export([renegotiate/2]).
+-export([renegotiate/2,
+        choose_tls_fsm/2]).
 
 %% gen_statem state functions
 -export([initial_hello/3,
@@ -141,11 +142,12 @@ init([Role, Sender, Host, Port, Socket, Options,  User, CbInfo]) ->
 	State1 = #state{static_env = #static_env{session_cache = Cache,
                                                  session_cache_cb = CacheCb
                                                 },
+                        connection_env = #connection_env{cert_key_pairs = CertKeyPairs},
                         ssl_options = SslOptions,
                         session = Session0} = ssl_gen_statem:ssl_config(State0#state.ssl_options, Role, State0),
         State = case Role of
                     client ->
-                        Session = ssl_session:client_select_session({Host, Port, SslOptions}, Cache, CacheCb, Session0),
+                        Session = ssl_session:client_select_session({Host, Port, SslOptions}, Cache, CacheCb, Session0, CertKeyPairs),
                         State1#state{session = Session};
                     server ->
                         State1
@@ -206,23 +208,20 @@ config_error(Type, Event, State) ->
 	    #state{}) ->
 		   gen_statem:state_function_result().
 %%--------------------------------------------------------------------
-hello(internal, #client_hello{extensions = Extensions} = Hello, 
-      #state{ssl_options = #{handshake := hello},
-             handshake_env = HsEnv,
+hello(internal, #client_hello{extensions = Extensions},
+      #state{static_env = #static_env{role = server},
+             handshake_env = #handshake_env{continue_status = pause},
+             start_or_recv_from = From} = State) ->
+    {next_state, user_hello, State#state{start_or_recv_from = undefined},
+     [{postpone, true}, {reply, From, {ok, Extensions}}]};
+hello(internal, #server_hello{extensions = Extensions},
+      #state{handshake_env = #handshake_env{continue_status = pause},
+             static_env = #static_env{role = client},
              start_or_recv_from = From} = State) ->
-    {next_state, user_hello, State#state{start_or_recv_from = undefined,
-                                         handshake_env = HsEnv#handshake_env{hello = Hello}},
-     [{reply, From, {ok, Extensions}}]};
-hello(internal, #server_hello{extensions = Extensions} = Hello,
-      #state{ssl_options = #{
-                             handshake := hello},
-             handshake_env = HsEnv,
-             start_or_recv_from = From} = State) ->   
     {next_state, user_hello,
-     State#state{start_or_recv_from = undefined,
-                 handshake_env = HsEnv#handshake_env{
-                                   hello = Hello}}, [{reply, From, {ok, Extensions}}]};
-hello(internal, #client_hello{client_version = ClientVersion} = Hello, #state{connection_env = CEnv} = State0) ->
+     State#state{start_or_recv_from = undefined}, [{postpone, true}, {reply, From, {ok, Extensions}}]};
+hello(internal, #client_hello{client_version = ClientVersion} = Hello,
+      #state{static_env = #static_env{role = server}, connection_env = CEnv} = State0) ->
     try
         #state{ssl_options = SslOpts} = State1 = tls_dtls_connection:handle_sni_extension(State0, Hello),
         case choose_tls_fsm(SslOpts, Hello) of
@@ -343,22 +342,26 @@ connection(internal, #hello_request{},
 	   #state{static_env = #static_env{role = client,
                                            host = Host,
                                            port = Port,
+                                           cert_db = CertDbHandle,
+                                           cert_db_ref = CertDbRef,
                                            session_cache = Cache,
                                            session_cache_cb = CacheCb},
                   handshake_env = #handshake_env{
-                      renegotiation = {Renegotiation, peer},
-                      ocsp_stapling_state = OcspState},
-		  session = #session{own_certificates = OwnCerts} = Session0,
+                                     renegotiation = {Renegotiation, peer},
+                                     ocsp_stapling_state = OcspState},
+                  connection_env = #connection_env{cert_key_pairs = CertKeyPairs},
+		  session = Session0,
 		  ssl_options = SslOpts, 
                   protocol_specific = #{sender := Pid},
 		  connection_states = ConnectionStates} = State0) ->
     try tls_sender:peer_renegotiate(Pid) of
         {ok, Write} ->
-            Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, Session0),
+            Session = ssl_session:client_select_session({Host, Port, SslOpts}, Cache, CacheCb, Session0, CertKeyPairs),
             Hello = tls_handshake:client_hello(Host, Port, ConnectionStates, SslOpts,
                                                Session#session.session_id,
-                                               Renegotiation, OwnCerts, undefined,
-                                               undefined, maps:get(ocsp_nonce, OcspState, undefined)),
+                                               Renegotiation, undefined,
+                                               undefined, maps:get(ocsp_nonce, OcspState, undefined),
+                                               CertDbHandle, CertDbRef),
             {State, Actions} = tls_gen_connection:send_handshake(Hello, 
                                                                  State0#state{connection_states = 
                                                                                   ConnectionStates#{current_write => Write},
@@ -371,16 +374,19 @@ connection(internal, #hello_request{},
 connection(internal, #hello_request{},
 	   #state{static_env = #static_env{role = client,
                                            host = Host,
-                                           port = Port},
+                                           port = Port,
+                                           cert_db = CertDbHandle,
+                                           cert_db_ref = CertDbRef
+                                          },
                   handshake_env = #handshake_env{
                       renegotiation = {Renegotiation, _},
                       ocsp_stapling_state = OcspState},
-		  session = #session{own_certificates = OwnCerts},
 		  ssl_options = SslOpts, 
 		  connection_states = ConnectionStates} = State0) ->
     Hello = tls_handshake:client_hello(Host, Port, ConnectionStates, SslOpts,
-                                       <<>>, Renegotiation, OwnCerts, undefined,
-                                       undefined, maps:get(ocsp_nonce, OcspState, undefined)),
+                                       <<>>, Renegotiation, undefined,
+                                       undefined, maps:get(ocsp_nonce, OcspState, undefined),
+                                      CertDbHandle, CertDbRef),
 
     {State, Actions} = tls_gen_connection:send_handshake(Hello, State0),
     tls_gen_connection:next_event(hello, no_record, State, Actions);
@@ -503,8 +509,8 @@ handle_client_hello(#client_hello{client_version = ClientVersion} = Hello, State
                               renegotiation = {Renegotiation, _},
                               negotiated_protocol = CurrentProtocol,
                               sni_guided_cert_selection = SNICertSelection} = HsEnv,
-           connection_env = CEnv,
-           session = #session{own_certificates = OwnCerts} = Session0,
+           connection_env = #connection_env{cert_key_pairs = CertKeyPairs} = CEnv,
+           session = Session0,
            ssl_options = SslOpts} = State,
     SessionTracker = proplists:get_value(session_id_tracker, Trackers),
     {Version, {Type, Session},
@@ -512,7 +518,7 @@ handle_client_hello(#client_hello{client_version = ClientVersion} = Hello, State
         tls_handshake:hello(Hello,
                             SslOpts,
                             {SessionTracker, Session0,
-                             ConnectionStates0, OwnCerts, KeyExAlg},
+                             ConnectionStates0, CertKeyPairs, KeyExAlg},
                             Renegotiation),
     Protocol = case Protocol0 of
                    undefined -> CurrentProtocol;
